from typing import Dict, List

import collections
import json
import os
import yaml

from copy import deepcopy
from pathlib import Path as Path


LOADER = {
    ".json": json.loads,
    ".yml": yaml.safe_load,
    ".yaml": yaml.safe_load,
}


def deep_merge(dict1, dict2):
    """ Return a new dictionary by merging two dictionaries recursively. """

    result = deepcopy(dict1)

    for key, value in dict2.items():
        if isinstance(value, collections.Mapping):
            result[key] = deep_merge(result.get(key, {}), value)
        else:
            result[key] = deepcopy(dict2[key])

    return result


def expand_environment_variables(contents: str) -> str:
    return os.path.expandvars(contents)


def load_config(path: Path, expand_envars=True, must_exist=False) -> Dict:
    if path.exists():
        contents = path.read_text()
        if expand_envars:
            contents = expand_environment_variables(contents)
        return LOADER[path.suffix](contents)
    elif must_exist:
        raise IOError(f"{path} does not exist!")
    else:
        return {}



import click


@click.group(short_help="Manage local accounts")
def cli():
    """
    Command-line helper for managing local accounts. You can unlock local accounts from
    scripts or the console using the accounts.load method.
    """


# Different name because `list` is a keyword
@cli.command(name="list", short_help="List available accounts")
def _list():
    pass


@cli.command(short_help="Add a new account by entering a private key")
@click.argument("id")
def new(id):
    pass


@cli.command(short_help="Add a new account with a random private key")
@click.argument("id")
def generate(id):
    click.echo("Generating a new private key...")


# Different name because `import` is a keyword
@cli.command(name="import", short_help="Import a new account via a keystore file")
@click.argument("id")
@click.argument("path", type=click.Path(exists=True, dir_okay=False, allow_dash=True))
def _import(id, path):
    pass


@cli.command(short_help="Export an existing account keystore file")
@click.argument("id")
@click.argument("path", type=click.Path(writable=True, allow_dash=True))
def export(id, path):
    pass


@cli.command(short_help="Change the password of an existing account")
@click.argument("id")
def password(id):
    pass


@cli.command(short_help="Delete an existing account")
@click.argument("id")
def delete(id):
    pass



#!/usr/bin/env python
from setuptools import find_packages, setup


extras_require = {
    "lint": [
        "black",
    ],
}
extras_require["dev"] = [
    "IPython",
] + extras_require["lint"]

setup(
    name="eth-ape",
    version="0.0.0-alpha.0",
    description="",
    long_description="",
    url="https://apeworx.io",
    author="ApeWorX Team",
    author_email="admin@apeworx.io",
    packages=find_packages(),
    install_requires=[
        "click>=7.1.2",
        "pyyaml>=0.2.5",
    ],
    entry_points={
        "console_scripts": ["ape=ape._cli.__init__:cli"],
    },
    include_package_data=True,
    python_requires=">=3.6,<4",
    extras_require=extras_require,
    classifiers=["Development Status :: 1 - Planning"],
)



from pathlib import Path

import click

cli_folder = Path(__file__).parent.absolute()


class ApeCLI(click.MultiCommand):
    def list_commands(self, ctx):
        return list(sorted(f.stem for f in cli_folder.glob("*.py") if f.stem != "__init__"))

    def get_command(self, ctx, name):
        ns = {}
        fn = cli_folder / Path(f"{name}.py")
        code = compile(fn.read_text(), fn, "exec")
        eval(code, ns, ns)
        return ns["cli"]


@click.command(cls=ApeCLI, context_settings=dict(help_option_names=["-h", "--help"]))
@click.version_option(message="%(version)s")
def cli():
    pass
